@@ -20,11 +20,6 @@ const ACVP_IMAGE = {
   path: "automation/taskcluster/docker-acvp"
 };
 
-const ECCKIILA_IMAGE = {
-  name: "ecckiila",
-  path: "automation/taskcluster/docker-ecckiila"
-};
-
 const CLANG_FORMAT_IMAGE = {
   name: "clang-format",
   path: "automation/taskcluster/docker-clang-format"
@@ -40,12 +35,6 @@ const FUZZ_IMAGE = {
   path: "automation/taskcluster/docker-fuzz"
 };
 
-// Bug 1488148 - temporary image for fuzzing 32-bit builds.
-const FUZZ_IMAGE_32 = {
-  name: "fuzz32",
-  path: "automation/taskcluster/docker-fuzz32"
-};
-
 const WINDOWS_CHECKOUT_CMD =
   "bash -c \"hg clone -r $NSS_HEAD_REVISION $NSS_HEAD_REPOSITORY nss || " +
     "(sleep 2; hg clone -r $NSS_HEAD_REVISION $NSS_HEAD_REPOSITORY nss) || " +
@@ -783,7 +772,7 @@ async function scheduleFuzzing32() {
     features: ["allowPtrace"],
     platform: "linux32",
     collection: "fuzz",
-    image: FUZZ_IMAGE_32
+    image: FUZZ_IMAGE
   };
 
   // Build base definition.
@@ -1180,17 +1169,6 @@ async function scheduleTools() {
     ]
   }));
   
-  queue.scheduleTask(merge(base, {
-    symbol: "ecckiila",
-    name: "ecckiila",
-    image: ECCKIILA_IMAGE,
-    command: [
-      "/bin/bash",
-      "-c",
-      "bin/checkout.sh && bin/ecckiila.sh && bin/run.sh"
-    ]
-  }));
-
   queue.scheduleTask(merge(base, {
     symbol: "Coverage",
     name: "Coverage",
