@@ -26,6 +26,7 @@
 #define SEC_CT_CERTIFICATE_REQUEST "certificate-request"
 #define SEC_CT_CERTIFICATE_ID "certificate-identity"
 #define SEC_CT_PKCS7 "pkcs7"
+#define SEC_CT_PKCS12 "pkcs12"
 #define SEC_CT_CRL "crl"
 #define SEC_CT_NAME "name"
 
@@ -235,10 +236,8 @@ extern void SECU_PrintTrustFlags(FILE *out, CERTCertTrust *trust, char *m,
 extern int SECU_PrintSubjectPublicKeyInfo(FILE *out, SECItem *der, char *m,
                                           int level);
 
-#ifdef HAVE_EPV_TEMPLATE
 /* Dump contents of private key */
 extern int SECU_PrintPrivateKey(FILE *out, SECItem *der, char *m, int level);
-#endif
 
 /* Dump contents of an RSA public key */
 extern void SECU_PrintRSAPublicKey(FILE *out, SECKEYPublicKey *pk, char *m, int level);
@@ -253,7 +252,8 @@ extern int SECU_PrintFingerprints(FILE *out, SECItem *derCert, char *m,
 /* Pretty-print any PKCS7 thing */
 extern int SECU_PrintPKCS7ContentInfo(FILE *out, SECItem *der, char *m,
                                       int level);
-
+/* Pretty-print a pkcs12 file */
+extern SECStatus SECU_PrintPKCS12(FILE *out, const SECItem *der, char *m, int level);
 /* Init PKCS11 stuff */
 extern SECStatus SECU_PKCS11Init(PRBool readOnly);
 
@@ -368,6 +368,22 @@ SECU_FindCrlIssuer(CERTCertDBHandle *dbHandle, SECItem *subject,
 typedef SECStatus (*EXTEN_EXT_VALUE_ENCODER)(PLArenaPool *extHandleArena,
                                              void *value, SECItem *encodedValue);
 
+#define DECL_EXTEN_EXT_VALUE_ENCODER(mmm)                                \
+    SECStatus EXTEN_EXT_VALUE_ENCODER_##mmm(PLArenaPool *extHandleArena, \
+                                            void *value, SECItem *encodedValue);
+
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeAltNameExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeAuthKeyID);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeBasicConstraintValue);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeCRLDistributionPoints);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeCertPoliciesExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeInfoAccessExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeInhibitAnyExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeNameConstraintsExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodePolicyConstraintsExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodePolicyMappingExtension);
+DECL_EXTEN_EXT_VALUE_ENCODER(CERT_EncodeSubjectKeyID);
+
 /* Encodes and adds extensions to the CRL or CRL entries. */
 SECStatus
 SECU_EncodeAndAddExtensionValue(PLArenaPool *arena, void *extHandle,
