@@ -329,52 +329,98 @@ static const oidValDef curveOptList[] = {
 static const oidValDef hashOptList[] = {
     /* Hashes */
     { CIPHER_NAME("MD2"), SEC_OID_MD2,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("MD4"), SEC_OID_MD4,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("MD5"), SEC_OID_MD5,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("SHA1"), SEC_OID_SHA1,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("SHA224"), SEC_OID_SHA224,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("SHA256"), SEC_OID_SHA256,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("SHA384"), SEC_OID_SHA384,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE },
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("SHA512"), SEC_OID_SHA512,
-      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE }
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("SHA3-224"), SEC_OID_SHA3_224,
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("SHA3-256"), SEC_OID_SHA3_256,
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("SHA3-384"), SEC_OID_SHA3_384,
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("SHA3-512"), SEC_OID_SHA3_512,
+      NSS_USE_ALG_IN_SSL_KX | NSS_USE_ALG_IN_SIGNATURE | NSS_USE_ALG_IN_SMIME |
+          NSS_USE_ALG_IN_PKCS12 }
 };
 
 static const oidValDef macOptList[] = {
     /* MACs */
-    { CIPHER_NAME("HMAC-SHA1"), SEC_OID_HMAC_SHA1, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("HMAC-SHA224"), SEC_OID_HMAC_SHA224, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("HMAC-SHA256"), SEC_OID_HMAC_SHA256, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("HMAC-SHA384"), SEC_OID_HMAC_SHA384, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("HMAC-SHA512"), SEC_OID_HMAC_SHA512, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("HMAC-MD5"), SEC_OID_HMAC_MD5, NSS_USE_ALG_IN_SSL },
+    { CIPHER_NAME("HMAC-MD5"), SEC_OID_HMAC_MD5,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA1"), SEC_OID_HMAC_SHA1,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA224"), SEC_OID_HMAC_SHA224,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA256"), SEC_OID_HMAC_SHA256,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA384"), SEC_OID_HMAC_SHA384,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA512"), SEC_OID_HMAC_SHA512,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA3-224"), SEC_OID_HMAC_SHA3_224,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA3-256"), SEC_OID_HMAC_SHA3_256,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA3-384"), SEC_OID_HMAC_SHA3_384,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("HMAC-SHA3-512"), SEC_OID_HMAC_SHA3_512,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
 };
 
 static const oidValDef cipherOptList[] = {
     /* Ciphers */
-    { CIPHER_NAME("AES128-CBC"), SEC_OID_AES_128_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("AES192-CBC"), SEC_OID_AES_192_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("AES256-CBC"), SEC_OID_AES_256_CBC, NSS_USE_ALG_IN_SSL },
+    { CIPHER_NAME("AES128-CBC"), SEC_OID_AES_128_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("AES192-CBC"), SEC_OID_AES_192_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("AES256-CBC"), SEC_OID_AES_256_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("AES128-GCM"), SEC_OID_AES_128_GCM, NSS_USE_ALG_IN_SSL },
     { CIPHER_NAME("AES192-GCM"), SEC_OID_AES_192_GCM, NSS_USE_ALG_IN_SSL },
     { CIPHER_NAME("AES256-GCM"), SEC_OID_AES_256_GCM, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("CAMELLIA128-CBC"), SEC_OID_CAMELLIA_128_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("CAMELLIA192-CBC"), SEC_OID_CAMELLIA_192_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("CAMELLIA256-CBC"), SEC_OID_CAMELLIA_256_CBC, NSS_USE_ALG_IN_SSL },
+    { CIPHER_NAME("CAMELLIA128-CBC"), SEC_OID_CAMELLIA_128_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("CAMELLIA192-CBC"), SEC_OID_CAMELLIA_192_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("CAMELLIA256-CBC"), SEC_OID_CAMELLIA_256_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("CHACHA20-POLY1305"), SEC_OID_CHACHA20_POLY1305, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("SEED-CBC"), SEC_OID_SEED_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("DES-EDE3-CBC"), SEC_OID_DES_EDE3_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("DES-40-CBC"), SEC_OID_DES_40_CBC, NSS_USE_ALG_IN_SSL },
+    { CIPHER_NAME("SEED-CBC"), SEC_OID_SEED_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("DES-EDE3-CBC"), SEC_OID_DES_EDE3_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("DES-40-CBC"), SEC_OID_DES_40_CBC,
+      NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SMIME | NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("DES-CBC"), SEC_OID_DES_CBC, NSS_USE_ALG_IN_SSL },
     { CIPHER_NAME("NULL-CIPHER"), SEC_OID_NULL_CIPHER, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("RC2"), SEC_OID_RC2_CBC, NSS_USE_ALG_IN_SSL },
-    { CIPHER_NAME("RC4"), SEC_OID_RC4, NSS_USE_ALG_IN_SSL },
+    { CIPHER_NAME("RC2"), SEC_OID_RC2_CBC, NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
+    { CIPHER_NAME("RC2-40-CBC"), SEC_OID_RC2_40_CBC, NSS_USE_ALG_IN_SMIME },
+    { CIPHER_NAME("RC2-64-CBC"), SEC_OID_RC2_64_CBC, NSS_USE_ALG_IN_SMIME },
+    { CIPHER_NAME("RC2-128-CBC"), SEC_OID_RC2_128_CBC, NSS_USE_ALG_IN_SMIME },
+    { CIPHER_NAME("RC4"), SEC_OID_RC4, NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_PKCS12 },
     { CIPHER_NAME("IDEA"), SEC_OID_IDEA_CBC, NSS_USE_ALG_IN_SSL },
 };
 
@@ -392,6 +438,14 @@ static const oidValDef kxOptList[] = {
     { CIPHER_NAME("ECDH-RSA"), SEC_OID_TLS_ECDH_RSA, NSS_USE_ALG_IN_SSL_KX },
 };
 
+static const oidValDef smimeKxOptList[] = {
+    /* Key exchange */
+    { CIPHER_NAME("RSA-PKCS"), SEC_OID_PKCS1_RSA_ENCRYPTION, NSS_USE_ALG_IN_SMIME_KX },
+    { CIPHER_NAME("RSA-OAEP"), SEC_OID_PKCS1_RSA_OAEP_ENCRYPTION, NSS_USE_ALG_IN_SMIME_KX },
+    { CIPHER_NAME("ECDH"), SEC_OID_ECDH_KEA, NSS_USE_ALG_IN_SMIME_KX },
+    { CIPHER_NAME("DH"), SEC_OID_X942_DIFFIE_HELMAN_KEY, NSS_USE_ALG_IN_SMIME_KX },
+};
+
 static const oidValDef signOptList[] = {
     /* Signatures */
     { CIPHER_NAME("DSA"), SEC_OID_ANSIX9_DSA_SIGNATURE,
@@ -418,7 +472,8 @@ static const algListsDef algOptLists[] = {
     { hashOptList, PR_ARRAY_SIZE(hashOptList), "HASH", PR_FALSE },
     { macOptList, PR_ARRAY_SIZE(macOptList), "MAC", PR_FALSE },
     { cipherOptList, PR_ARRAY_SIZE(cipherOptList), "CIPHER", PR_FALSE },
-    { kxOptList, PR_ARRAY_SIZE(kxOptList), "OTHER-KX", PR_FALSE },
+    { kxOptList, PR_ARRAY_SIZE(kxOptList), "SSL-KX", PR_FALSE },
+    { smimeKxOptList, PR_ARRAY_SIZE(smimeKxOptList), "SMIME-KX", PR_TRUE },
     { signOptList, PR_ARRAY_SIZE(signOptList), "OTHER-SIGN", PR_FALSE },
 };
 
@@ -441,6 +496,8 @@ static const optionFreeDef keySizeFlagsList[] = {
     { CIPHER_NAME("KEY-SIZE-SSL"), NSS_KEY_SIZE_POLICY_SSL_FLAG },
     { CIPHER_NAME("KEY-SIZE-SIGN"), NSS_KEY_SIZE_POLICY_SIGN_FLAG },
     { CIPHER_NAME("KEY-SIZE-VERIFY"), NSS_KEY_SIZE_POLICY_VERIFY_FLAG },
+    { CIPHER_NAME("KEY-SIZE-SMIME"), NSS_KEY_SIZE_POLICY_SMIME_FLAG },
+    { CIPHER_NAME("KEY-SIZE-ALL"), NSS_KEY_SIZE_POLICY_ALL_FLAGS },
 };
 
 static const optionFreeDef freeOptList[] = {
@@ -464,21 +521,46 @@ static const policyFlagDef policyFlagList[] = {
     { CIPHER_NAME("SSL"), NSS_USE_ALG_IN_SSL },
     { CIPHER_NAME("SSL-KEY-EXCHANGE"), NSS_USE_ALG_IN_SSL_KX },
     /* add other key exhanges in the future */
-    { CIPHER_NAME("KEY-EXCHANGE"), NSS_USE_ALG_IN_SSL_KX },
+    { CIPHER_NAME("KEY-EXCHANGE"), NSS_USE_ALG_IN_KEY_EXCHANGE },
     { CIPHER_NAME("CERT-SIGNATURE"), NSS_USE_ALG_IN_CERT_SIGNATURE },
-    { CIPHER_NAME("CMS-SIGNATURE"), NSS_USE_ALG_IN_CMS_SIGNATURE },
+    { CIPHER_NAME("CMS-SIGNATURE"), NSS_USE_ALG_IN_SMIME_SIGNATURE },
+    { CIPHER_NAME("SMIME-SIGNATURE"), NSS_USE_ALG_IN_SMIME_SIGNATURE },
     { CIPHER_NAME("ALL-SIGNATURE"), NSS_USE_ALG_IN_SIGNATURE },
+    { CIPHER_NAME("PKCS12"), NSS_USE_ALG_IN_PKCS12 },
+    /* only use in allow */
+    { CIPHER_NAME("PKCS12-LEGACY"), NSS_USE_ALG_IN_PKCS12_DECRYPT },
+    /* only use in disallow */
+    { CIPHER_NAME("PKCS12-ENCRYPT"), NSS_USE_ALG_IN_PKCS12_ENCRYPT },
+    { CIPHER_NAME("SMIME"), NSS_USE_ALG_IN_SMIME },
+    /* only use in allow, enable */
+    { CIPHER_NAME("SMIME-LEGACY"), NSS_USE_ALG_IN_SMIME_LEGACY },
+    /* only use in disallow, disable */
+    { CIPHER_NAME("SMIME-ENCRYPT"), NSS_USE_ALG_IN_SMIME_ENCRYPT },
+    { CIPHER_NAME("SMIME-KEY-EXCHANGE"), NSS_USE_ALG_IN_SMIME_KX },
+    /* only use in allow */
+    { CIPHER_NAME("SMIME-KEY-EXCHANGE-LEGACY"), NSS_USE_ALG_IN_SMIME_KX_LEGACY },
+    /* only use in disallow */
+    { CIPHER_NAME("SMIME-KEY-EXCHANGE-ENCRYPT"), NSS_USE_ALG_IN_SMIME_KX_ENCRYPT },
     /* sign turns off all signatures, but doesn't change the
-     * allowance for specific sigantures... for example:
-     * disallow=sha256/all allow=sha256/signature doesn't allow
-     * cert-sigantures, where disallow=sha256/all allow=sha256/all-signature
-     * does.
-     * however, disallow=sha356/signature and disallow=sha256/all-siganture are
-     * equivalent in effect */
+     * allowance for specific signatures... for example:
+     *     disallow=sha256/all allow=sha256/signature
+     * doesn't allow cert-signatures or sime-signatures, where
+     *     disallow=sha256/all allow=sha256/all-signature
+     * does. however,
+     *     disallow=sha256/signature
+     * and
+     *     disallow=sha256/all-signature
+     * are equivalent in effect */
     { CIPHER_NAME("SIGNATURE"), NSS_USE_ALG_IN_ANY_SIGNATURE },
+    /* enable/allow algorithms for legacy (read/verify)operations */
+    { CIPHER_NAME("LEGACY"), NSS_USE_ALG_IN_PKCS12_DECRYPT |
+                                 NSS_USE_ALG_IN_SMIME_LEGACY |
+                                 NSS_USE_ALG_IN_SMIME_KX_LEGACY },
     /* enable/disable everything */
     { CIPHER_NAME("ALL"), NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SSL_KX |
-                              NSS_USE_ALG_IN_SIGNATURE },
+                              NSS_USE_ALG_IN_PKCS12 | NSS_USE_ALG_IN_SMIME |
+                              NSS_USE_ALG_IN_SIGNATURE |
+                              NSS_USE_ALG_IN_SMIME_KX },
     { CIPHER_NAME("NONE"), 0 }
 };
 
@@ -560,6 +642,9 @@ secmod_getPolicyOptValue(const char *policyValue, int policyValueLength,
         *result = val;
         return SECSuccess;
     }
+    if (policyValueLength == 0) {
+        return SECFailure;
+    }
     /* handle any ssl strings */
     for (i = 0; i < PR_ARRAY_SIZE(sslOptList); i++) {
         if (policyValueLength == sslOptList[i].name_size &&
@@ -572,7 +657,7 @@ secmod_getPolicyOptValue(const char *policyValue, int policyValueLength,
     /* handle key_size flags. Each flag represents a bit, which
      * gets or'd together. They can be separated by , | or + */
     val = 0;
-    while (*policyValue) {
+    while (policyValueLength > 0) {
         PRBool found = PR_FALSE;
         for (i = 0; i < PR_ARRAY_SIZE(keySizeFlagsList); i++) {
             if (PORT_Strncasecmp(keySizeFlagsList[i].name, policyValue,
@@ -580,6 +665,7 @@ secmod_getPolicyOptValue(const char *policyValue, int policyValueLength,
                 val |= keySizeFlagsList[i].option;
                 found = PR_TRUE;
                 policyValue += keySizeFlagsList[i].name_size;
+                policyValueLength -= keySizeFlagsList[i].name_size;
                 break;
             }
         }
@@ -588,6 +674,7 @@ secmod_getPolicyOptValue(const char *policyValue, int policyValueLength,
         }
         if (*policyValue == ',' || *policyValue == '|' || *policyValue == '+') {
             policyValue++;
+            policyValueLength--;
         }
     }
     *result = val;
@@ -607,6 +694,59 @@ typedef enum {
     NSS_ENABLE
 } NSSPolicyOperation;
 
+/* Enable/Disable only apply to SSL cipher suites and S/MIME symetric algorithms.
+ * Enable/Disable is implemented by clearing the DEFAULT_NOT_VALID
+ * flag, then setting the NSS_USE_DEFAULT_SSL_ENABLE and
+ * NSS_USE_DEFAULT_SMIME_ENABLE flags to the correct value. The ssl
+ * policy code will then sort out what to set based on ciphers and
+ * cipher suite values and the smime policy code will sort
+ * out which ciphers to include in capabilities based on these values */
+static SECStatus
+secmod_setDefault(SECOidTag oid, NSSPolicyOperation operation,
+                  PRUint32 value)
+{
+    SECStatus rv = SECSuccess;
+    PRUint32 policy;
+    PRUint32 useDefault = 0;
+    PRUint32 set = 0;
+    /* we always clear the default not valid flag as this operation will
+     * make the defaults valid */
+    PRUint32 clear = NSS_USE_DEFAULT_NOT_VALID;
+
+    /* what values are we trying to change */
+    /* if either SSL or SSL_KX is set, enable SSL */
+    if (value & (NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SSL_KX)) {
+        useDefault |= NSS_USE_DEFAULT_SSL_ENABLE;
+    }
+    /* only bulk ciphers are configured as enable in S/MIME, only
+     * enable them if both SMIME bits are set */
+    if ((value & NSS_USE_ALG_IN_SMIME) == NSS_USE_ALG_IN_SMIME) {
+        useDefault |= NSS_USE_DEFAULT_SMIME_ENABLE;
+    }
+
+    /* on disable we clear, on enable we set */
+    if (operation == NSS_DISABLE) {
+        clear |= useDefault;
+    } else {
+        /* we also turn the cipher on by policy if we enable it,
+         * so include the policy bits */
+        set |= value | useDefault;
+    }
+
+    /* if we haven't set the not valid flag yet, then we need to
+     * clear any of the other bits we aren't actually setting as well.
+     */
+    rv = NSS_GetAlgorithmPolicy(oid, &policy);
+    if (rv != SECSuccess) {
+        return rv;
+    }
+    if (policy & NSS_USE_DEFAULT_NOT_VALID) {
+        clear |= ((NSS_USE_DEFAULT_SSL_ENABLE | NSS_USE_DEFAULT_SMIME_ENABLE) &
+                  ~set);
+    }
+    return NSS_SetAlgorithmPolicy(oid, set, clear);
+}
+
 /* apply the operator specific policy */
 SECStatus
 secmod_setPolicyOperation(SECOidTag oid, NSSPolicyOperation operation,
@@ -622,25 +762,9 @@ secmod_setPolicyOperation(SECOidTag oid, NSSPolicyOperation operation,
             /* set the requested policy bits */
             rv = NSS_SetAlgorithmPolicy(oid, value, 0);
             break;
-        /* enable/disable only apply to SSL cipher suites (future S/MIME).
-         * Enable/disable is implemented by clearing the DEFAULT_NOT_VALID
-         * flag, then setting the NSS_USE_DEFAULT_SSL_ENABLE flag to the
-         * correct value. The ssl policy code will then sort out what to
-         * set based on ciphers and cipher suite values.*/
         case NSS_DISABLE:
-            if (value & (NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SSL_KX)) {
-                /* clear not valid and enable */
-                rv = NSS_SetAlgorithmPolicy(oid, 0,
-                                            NSS_USE_DEFAULT_NOT_VALID |
-                                                NSS_USE_DEFAULT_SSL_ENABLE);
-            }
-            break;
         case NSS_ENABLE:
-            if (value & (NSS_USE_ALG_IN_SSL | NSS_USE_ALG_IN_SSL_KX)) {
-                /* set enable, clear not valid. NOTE: enable implies allow! */
-                rv = NSS_SetAlgorithmPolicy(oid, value | NSS_USE_DEFAULT_SSL_ENABLE,
-                                            NSS_USE_DEFAULT_NOT_VALID);
-            }
+            rv = secmod_setDefault(oid, operation, value);
             break;
         default:
             PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
@@ -819,7 +943,7 @@ secmod_sanityCheckCryptoPolicy(void)
             if ((algOpt->val & NSS_USE_ALG_IN_SSL_KX) && (value & NSS_USE_ALG_IN_SSL_KX)) {
                 ++num_kx_enabled;
                 anyEnabled = PR_TRUE;
-                fprintf(stderr, "NSS-POLICY-INFO: %s is enabled for KX\n", algOpt->name);
+                fprintf(stderr, "NSS-POLICY-INFO: %s is enabled for SSL-KX\n", algOpt->name);
             }
             if ((algOpt->val & NSS_USE_ALG_IN_SSL) && (value & NSS_USE_ALG_IN_SSL)) {
                 ++num_ssl_enabled;
